@@ -1915,7 +1915,6 @@ static gboolean remmina_plugin_vnc_on_size_allocate(GtkWidget *widget, GtkAlloca
 		sprintf(str, "DEBUG: %d x %d", alloc->width, alloc->height);
 		TRACE_CALL(str);
 		if (gpdata->client){
-			rfbClient *cl;
 			SendExtDesktopSize(gpdata->client, alloc->width, alloc->height);
 		}
 	}
@@ -1962,6 +1961,7 @@ static void remmina_plugin_vnc_init(RemminaProtocolWidget *gp)
 	TRACE_CALL(__func__);
 	RemminaPluginVncData *gpdata;
 	gint flags;
+	gdouble aspect_ratio;
 
 	gpdata = g_new0(RemminaPluginVncData, 1);
 	g_object_set_data_full(G_OBJECT(gp), "plugin-data", gpdata, g_free);
@@ -1974,7 +1974,19 @@ static void remmina_plugin_vnc_init(RemminaProtocolWidget *gp)
 
 	gpdata->drawing_area = gtk_drawing_area_new();
 	gtk_widget_show(gpdata->drawing_area);
-	gtk_container_add(GTK_CONTAINER(gp), gpdata->drawing_area);
+
+	aspect_ratio = remmina_plugin_service->file_get_double(remminafile, "aspect_ratio", 0);
+	if (aspect_ratio > 0){
+		GtkWidget* aspectframe = gtk_aspect_frame_new(NULL, 0, 0, aspect_ratio, FALSE);
+
+		gtk_frame_set_shadow_type(GTK_FRAME(aspectframe), GTK_SHADOW_NONE);
+		gtk_widget_show(aspectframe);
+		gtk_container_add(GTK_CONTAINER(aspectframe), gpdata->drawing_area);
+		gtk_container_add(GTK_CONTAINER(gp), aspectframe);
+	}
+	else{
+		gtk_container_add(GTK_CONTAINER(gp), gpdata->drawing_area);
+	}
 
 	gtk_widget_add_events(
 		gpdata->drawing_area,
@@ -2047,6 +2059,13 @@ static gchar vnciport_tooltip[] =
 	   "    Remmina, e.g. with x11vnc:\n"
 	   "    x11vnc -display :0 -connect 192.168.1.36:8888");
 
+static gchar aspect_ratio_tooltip[] =
+	N_("Lock the aspect ratio when dynamic resolution is enabled:\n"
+	   "\n"
+	   "  • The aspect ratio should be entered as a decimal number, e.g. 1.777\n"
+	   "  • 16:9 corresponds roughly to 1.7777, 4:3 corresponds roughly to 1.333\n"
+	   "  • The default value of 0 does not enforce any aspect ratio");	   
+
 static gchar vncencodings_tooltip[] =
 	N_("Overriding the pre-set VNC encoding quality:\n"
 	   "\n"
@@ -2108,6 +2127,7 @@ static const RemminaProtocolSetting remmina_plugin_vnci_basic_settings[] =
 static const RemminaProtocolSetting remmina_plugin_vnc_advanced_settings[] =
 {
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,  "encodings",		 N_("Override pre-set VNC encodings"),	        FALSE, NULL, vncencodings_tooltip },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,  "aspect_ratio",		 N_("Dynamic resolution enforced aspec ratio"),	        FALSE, NULL, aspect_ratio_tooltip },
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "tightencoding", N_("Force tight encoding"),			TRUE, NULL, N_("Enabling this may help when the remote desktop looks scrambled") },
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablesmoothscrolling", N_("Disable smooth scrolling"),		FALSE, NULL, NULL },
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablepasswordstoring", N_("Forget passwords after use"),		TRUE,  NULL, NULL },
