@@ -3,6 +3,7 @@
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
  * Copyright (C) 2016-2023 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2023-2024 Hiroyuki Tanaka, Sunil Bhat
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -121,12 +122,14 @@ struct _RemminaFileEditorPriv {
 	GtkWidget *		behavior_precommand_entry;
 	GtkWidget *		behavior_postcommand_entry;
 	GtkWidget *		behavior_lock_check;
+	GtkWidget *		behavior_disconnect;
 
 	GtkWidget *		ssh_tunnel_enabled_check;
 	GtkWidget *		ssh_tunnel_loopback_check;
 	GtkWidget *		ssh_tunnel_server_default_radio;
 	GtkWidget *		ssh_tunnel_server_custom_radio;
 	GtkWidget *		ssh_tunnel_server_entry;
+	GtkWidget *		ssh_tunnel_command_entry;
 	GtkWidget *		ssh_tunnel_auth_agent_radio;
 	GtkWidget *		ssh_tunnel_auth_password_radio;
 	GtkWidget *		ssh_tunnel_auth_password;
@@ -360,17 +363,6 @@ static GtkWidget *remmina_file_editor_create_notebook_tab(RemminaFileEditor *gfe
 	return grid;
 }
 
-#ifdef HAVE_LIBSSH
-
-static void remmina_file_editor_ssh_tunnel_server_custom_radio_on_toggled(GtkToggleButton *togglebutton, RemminaFileEditor *gfe)
-{
-	TRACE_CALL(__func__);
-	gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_server_entry),
-				 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gfe->priv->ssh_tunnel_enabled_check)) &&
-				 (gfe->priv->ssh_tunnel_server_custom_radio == NULL ||
-				  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gfe->priv->ssh_tunnel_server_custom_radio))));
-}
-
 
 static void remmina_file_editor_assistance_enabled_check_on_toggled(GtkToggleButton *togglebutton,
 								    RemminaFileEditor *gfe)
@@ -391,6 +383,18 @@ static void remmina_file_editor_assistance_enabled_check_on_toggled(GtkToggleBut
 	}
 }
 
+#ifdef HAVE_LIBSSH
+
+static void remmina_file_editor_ssh_tunnel_server_custom_radio_on_toggled(GtkToggleButton *togglebutton, RemminaFileEditor *gfe)
+{
+	TRACE_CALL(__func__);
+	gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_server_entry),
+				 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gfe->priv->ssh_tunnel_enabled_check)) &&
+				 (gfe->priv->ssh_tunnel_server_custom_radio == NULL ||
+				  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gfe->priv->ssh_tunnel_server_custom_radio))));
+}
+
+
 static void remmina_file_editor_ssh_tunnel_enabled_check_on_toggled(GtkToggleButton *togglebutton,
 								    RemminaFileEditor *gfe, RemminaProtocolSSHSetting ssh_setting)
 {
@@ -415,6 +419,8 @@ static void remmina_file_editor_ssh_tunnel_enabled_check_on_toggled(GtkToggleBut
 		gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_username_entry), enabled);
 		gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_auth_password), enabled);
 		gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_auth_combo), enabled);
+		gtk_widget_set_sensitive(GTK_WIDGET(gfe->priv->ssh_tunnel_command_entry), enabled);
+		
 		//}
 		g_free(p);
 	}
@@ -444,6 +450,12 @@ static void remmina_file_editor_ssh_tunnel_enabled_check_on_toggled(GtkToggleBut
 			gtk_entry_set_text(GTK_ENTRY(gfe->priv->ssh_tunnel_passphrase), cp ? cp : "");
 		}
 	}
+	if (gfe->priv->ssh_tunnel_command_entry) {
+		if (enabled && gtk_entry_get_text(GTK_ENTRY(gfe->priv->ssh_tunnel_command_entry))[0] == '\0') {
+			cp = remmina_file_get_string(priv->remmina_file, "ssh_tunnel_command");
+			gtk_entry_set_text(GTK_ENTRY(gfe->priv->ssh_tunnel_command_entry), cp ? cp : "");
+		}
+	}
 }
 
 #endif
@@ -670,7 +682,10 @@ static void remmina_file_editor_create_assistance(RemminaFileEditor *gfe, const
 	widget = gtk_entry_new();
 	gtk_widget_set_halign(widget, GTK_ALIGN_START);
 	gtk_widget_show(widget);
-	gtk_entry_set_text(GTK_ENTRY(widget), remmina_file_get_string(gfe->priv->remmina_file, "assistance_file"));
+	
+	if (remmina_file_get_string(gfe->priv->remmina_file, "assistance_file") != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(widget), remmina_file_get_string(gfe->priv->remmina_file, "assistance_file"));
+	}
 	gtk_grid_attach(GTK_GRID(grid), widget, 1, row+1, 1, 1);
 	gfe->priv->assistance_file = widget;
 
@@ -683,7 +698,10 @@ static void remmina_file_editor_create_assistance(RemminaFileEditor *gfe, const
 	widget = gtk_entry_new();
 	gtk_widget_set_halign(widget, GTK_ALIGN_START);
 	gtk_widget_show(widget);
-	gtk_entry_set_text(GTK_ENTRY(widget), remmina_file_get_string(gfe->priv->remmina_file, "assistance_pass"));
+
+	if (remmina_file_get_string(gfe->priv->remmina_file, "assistance_pass") != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(widget), remmina_file_get_string(gfe->priv->remmina_file, "assistance_pass"));
+	}
 	gtk_grid_attach(GTK_GRID(grid), widget, 1, row+2, 1, 1);
 	gfe->priv->assistance_password = widget;
 
@@ -1192,6 +1210,13 @@ static void remmina_file_editor_create_behavior_tab(RemminaFileEditor *gfe)
 	/* Autostart profile option */
 	priv->behavior_lock_check = remmina_file_editor_create_check(gfe, grid, 10, 1, _("Require password to connect or edit the profile"),
 								     remmina_file_get_int(priv->remmina_file, "profile-lock", FALSE), "profile-lock");
+
+									 /* Startup frame */
+	remmina_public_create_group(GTK_GRID(grid), _("Unexpected disconnect"), 12, 1, 2);
+
+	/* Autostart profile option */
+	priv->behavior_disconnect = remmina_file_editor_create_check(gfe, grid, 16, 1, _("Keep window from closing if not disconnected by Remmina"),
+								     remmina_file_get_int(priv->remmina_file, "disconnect-prompt", FALSE), "disconnect-prompt");
 }
 
 #ifdef HAVE_LIBSSH
@@ -1368,6 +1393,19 @@ static void remmina_file_editor_create_ssh_tunnel_tab(RemminaFileEditor *gfe, Re
 				   cs ? cs : "");
 	}
 
+
+	remmina_public_create_group(GTK_GRID(grid), _("SSH Command"), row, 2, 1);
+	row += 2;
+	priv->ssh_tunnel_command_entry =
+			remmina_file_editor_create_text(gfe, grid, row, 0,
+							_("Startup command"), NULL, "ssh_tunnel_command");
+	cs = remmina_file_get_string(priv->remmina_file, "ssh_tunnel_command");
+	gtk_entry_set_text(GTK_ENTRY(priv->ssh_tunnel_command_entry),
+				   cs ? cs : "");
+	row++;
+
+
+
 	remmina_file_editor_ssh_tunnel_enabled_check_on_toggled(NULL, gfe, ssh_setting);
 	gtk_widget_show_all(grid);
 	g_free(p);
@@ -1439,6 +1477,7 @@ static void remmina_file_editor_protocol_combo_on_changed(GtkComboBox *combo, Re
 	priv->ssh_tunnel_server_custom_radio = NULL;
 	priv->ssh_tunnel_server_entry = NULL;
 	priv->ssh_tunnel_username_entry = NULL;
+	priv->ssh_tunnel_command_entry = NULL;
 	priv->ssh_tunnel_auth_combo = NULL;
 	priv->ssh_tunnel_auth_password = NULL;
 	priv->ssh_tunnel_privatekey_chooser = NULL;
@@ -1467,6 +1506,8 @@ static void remmina_file_editor_save_behavior_tab(RemminaFileEditor *gfe)
 	remmina_file_set_int(priv->remmina_file, "enable-autostart", autostart_enabled);
 	gboolean lock_enabled = (priv->behavior_lock_check ? gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->behavior_lock_check)) : FALSE);
 	remmina_file_set_int(priv->remmina_file, "profile-lock", lock_enabled);
+	gboolean disconect_prompt = (priv->behavior_disconnect ? gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->behavior_disconnect)) : FALSE);
+	remmina_file_set_int(priv->remmina_file, "disconnect-prompt", disconect_prompt);
 }
 
 static void remmina_file_editor_save_ssh_tunnel_tab(RemminaFileEditor *gfe)
@@ -1524,6 +1565,11 @@ static void remmina_file_editor_save_ssh_tunnel_tab(RemminaFileEditor *gfe)
 		"ssh_tunnel_password",
 		(ssh_tunnel_enabled && (ssh_tunnel_auth == SSH_AUTH_PASSWORD)) ? gtk_entry_get_text(GTK_ENTRY(priv->ssh_tunnel_auth_password)) : NULL);
 
+	char* command = gtk_entry_get_text(GTK_ENTRY(priv->ssh_tunnel_command_entry));
+	remmina_file_set_string(
+		priv->remmina_file,
+		"ssh_tunnel_command", command);
+
 	remmina_file_set_string(
 		priv->remmina_file,
 		"ssh_tunnel_passphrase",
