@@ -363,8 +363,8 @@ static gboolean conn_closed(gpointer data)
 {
 	TRACE_CALL(__func__);
 	RemminaProtocolWidget *gp = (RemminaProtocolWidget *)data;
-
-	if (!gp->priv->user_disconnect && !gp->priv->has_error){
+	int disconnect_prompt = remmina_file_get_int(gp->priv->remmina_file, "disconnect-prompt", FALSE);
+	if (!gp->priv->user_disconnect && !gp->priv->has_error && disconnect_prompt){
 		const char* msg = "Plugin Disconnected";
 		if (gp->priv->has_error){
 			msg = remmina_protocol_widget_get_error_message(gp);
@@ -723,7 +723,7 @@ void remmina_protocol_widget_send_clip_strokes(GtkClipboard *clipboard, const gc
 	return;
 }
 
-void remmina_protocol_widget_send_clipboard(RemminaProtocolWidget *gp, GtkMenuItem *widget)
+void remmina_protocol_widget_send_clipboard(RemminaProtocolWidget *gp, GObject*widget)
 {
 	TRACE_CALL(__func__);
 	GtkClipboard *clipboard;
@@ -1168,6 +1168,44 @@ gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gi
 
 	g_ptr_array_add(gp->priv->ssh_tunnels, tunnel);
 
+
+	//try startup command
+	ssh_channel channel;
+	int rc;
+	const gchar* tunnel_command = remmina_file_get_string(gp->priv->remmina_file, "ssh_tunnel_command");
+	if (tunnel_command != NULL){
+		channel = ssh_channel_new(REMMINA_SSH(tunnel)->session);
+		if (channel == NULL) return g_strdup_printf("127.0.0.1:%i", remmina_pref.sshtunnel_port);
+
+		rc = ssh_channel_open_session(channel);
+		if (rc != SSH_OK)
+		{
+			ssh_channel_free(channel);
+			return g_strdup_printf("127.0.0.1:%i", remmina_pref.sshtunnel_port);
+		}
+		rc = ssh_channel_request_exec(channel, tunnel_command);
+		if (rc != SSH_OK)
+		{
+			ssh_channel_close(channel);
+			ssh_channel_free(channel);
+			return g_strdup_printf("127.0.0.1:%i", remmina_pref.sshtunnel_port);
+		}
+		struct timeval timeout = {10, 0};
+		ssh_channel channels[2];
+		channels[0] = channel;
+		channels[1] = NULL;
+		rc = ssh_channel_select(channels, NULL, NULL, &timeout);
+		if (rc == SSH_OK){
+			char buffer[256];
+			ssh_channel_read(channel, buffer, sizeof(buffer), 0);
+		}
+		
+		REMMINA_DEBUG("Ran startup command");
+		ssh_channel_close(channel);
+		ssh_channel_free(channel);
+	}
+
+
 	return g_strdup_printf("127.0.0.1:%i", remmina_pref.sshtunnel_port);
 
 #else
@@ -1571,6 +1609,9 @@ static gboolean remmina_protocol_widget_dialog_mt_setup(gpointer user_data)
 	mp = remmina_message_panel_new();
 
 	if (d->dtype == RPWDT_AUTH) {
+		if (d->pflags & REMMINA_MESSAGE_PANEL_FLAG_USERNAME) {
+			remmina_message_panel_field_set_string(mp, REMMINA_MESSAGE_PANEL_USERNAME, d->default_username);
+		}
 		remmina_message_panel_setup_auth(mp, authpanel_mt_cb, d, d->title, d->strpasswordlabel, d->pflags);
 		remmina_message_panel_field_set_string(mp, REMMINA_MESSAGE_PANEL_USERNAME, d->default_username);
 		if (d->pflags & REMMINA_MESSAGE_PANEL_FLAG_DOMAIN)
